#ifndef VEX_MOTOR_H
#define VEX_MOTOR_H

#include "brushed-motor.hpp"
#include "vex-encoder.hpp"

using namespace stm32f072xb;

/**
 * @brief TU DOIS LE METTRE SUR LES MEMES PWM PORT ET DIRECTION PORT POUR TOUS TES VEX MOTORS
 *
 * @tparam pwmPort
 * @tparam directionPort
 */
template <Port pwmPort, Port directionPort>
class VexMotor : public BrushedMotor<pwmPort, directionPort>, public VexEncoder {
   public:
	VexMotor(BrushedMotorInit& motorInit, VexEncoderInit& encoderInit)
	    : BrushedMotor<pwmPort, directionPort>(motorInit, 255), VexEncoder(encoderInit) {}

	void setSpeed(int16_t speed) {
		if (isAllowedToMove(speed)) {
			BrushedMotor<pwmPort, directionPort>::setSpeed(speed);
		} else {
			BrushedMotor<pwmPort, directionPort>::setSpeed(0);
		}
	}

   private:
	bool isAllowedToMove(int16_t speed) {
		const float minPosition = 0.05f;
		bool isAllow = true;
		float absolutePosition = abs(this->getPosition());

		// Prevent going foward if we are already at the maximum forward
		if (speed >= 0 && absolutePosition >= this->getMaxPosition()) {
			isAllow = false;
		} else if (speed <= 0 && absolutePosition < minPosition) {
			isAllow = false;
		}

		return isAllow;
	}
};

#endif
