#include <basic-timer.hpp>
using namespace stm32f072xb;

extern "C" void TIM6_DAC_IRQHandler() {
	SET_BIT(GPIOA->ODR, GPIO_ODR_5);  // Turn on LED, p.159
	CLEAR_BIT(TIM6->SR,
	          TIM_SR_UIF);  // TIM6 status register :: Clear interrupt (p. 557)
}

void setupGpio() {
	SET_BIT(RCC->AHBENR, RCC_AHBENR_GPIOAEN);  // Enable GPIOA (p. 120)
}

void setupLed() {
	CLEAR_BIT(GPIOA->MODER, GPIO_MODER_MODER5);
	SET_BIT(GPIOA->MODER, GPIO_MODER_MODER5_0);  // Output mode (p.157)
	CLEAR_BIT(GPIOA->OTYPER, GPIO_OTYPER_OT_5);  // Push-pull output (p.157)
	CLEAR_BIT(GPIOA->ODR, GPIO_ODR_5);           // Turn off LED, p.159
}

int main() {
	constexpr uint16_t autoReloadValue =
	    122;  // 8 000 000 / 65535 = 122.07 clk cycles to burn
	constexpr uint16_t prescalerValue = 0xFFFF;  // clk / 65535
	constexpr uint32_t interruptPriority = 0;    // 0 = highest priority

	setupGpio();
	setupLed();

	BasicTimerInit init{TIM6,
	                    Counter::DISABLE,
	                    UevUpdate::ENABLE,
	                    UpdateRequestSource::DISABLE,
	                    OnePulse::ENABLE,
	                    AutoReload::DISABLE,
	                    MasterModeSelection::RESET,
	                    interruptPriority};

	BasicTimer timer6{init};

	timer6.start(autoReloadValue, prescalerValue);

	while (true)
		;

	timer6.stop();
}
